#
# Barpanel "MENU" plugin
# 
# Copyright (C) 2005 - 2006, Federico Pelloni <federico.pelloni@gmail.com>
# Copyright (C) 2008 Job and Esther Technologies Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# TODO: 
#  - implement an optional FAM monitor to watch changes to menu

name           = "Menu"
description    = "A new-concept auto-generated menu focused on ease-of-use."

import barpanel, xdg.Menu, xdg.IconTheme, gtk, gtk.gdk, os, re, commands, traceback

theme = gtk.icon_theme_get_default()

class item(gtk.Button):
    class item_info:
        name = None
        description = None
        command = None
        icon = None
        terminal = False

    def __init__(self, obj):
        gtk.Button.__init__(self)
        self.info = self.item_info()
        self.info.name = obj.getName().replace('&', '&amp;')
        self.info.description = obj.getComment().replace('&', '&amp;')
        self.info.command = obj.getExec()
        self.info.terminal = obj.getTerminal()
        self.set_relief(gtk.RELIEF_NONE)
        cont = gtk.HBox(False, 10)
        cont.set_border_width(3)
        cont.show()
        self.add(cont)
        image = gtk.Image()
        try:
            name = obj.getIcon()
            if name[0] == '/' and os.path.exists(name):
                pb = gtk.gdk.pixbuf_new_from_file_at_size(name, 24, 24)
            else:
                name = name.rsplit('.', 1)[0]
                pb = theme.load_icon(name, 24, 0)
            w = pb.get_width()
            h = pb.get_height()
            if h != 24:
                pb = pb.scale_simple(24, 24, gtk.gdk.INTERP_BILINEAR)
            image.set_from_pixbuf(pb)
        except Exception, msg:
            image.set_from_stock(gtk.STOCK_EXECUTE, gtk.ICON_SIZE_LARGE_TOOLBAR)
   	    image.set_pixel_size(24)
        image.show()
        cont.pack_start(image, False, False)
        label = gtk.Label()
        label.show()
        label.set_property("xalign", 0)
        label.set_markup("<b>%s</b>\n%s" % (self.info.name, self.info.description))
        cont.pack_start(label, True, True)

class separator(gtk.HSeparator):
    def __init__(self):
        gtk.HSeparator.__init__(self)

class menu(gtk.ToggleButton):
    def __init__(self, obj, panel, plugin):
        self.panel = panel
        self.plugin = plugin
        gtk.ToggleButton.__init__(self)
        self.set_relief(gtk.RELIEF_NONE)
        self.set_name("EdgeButton")
        self.show()
        self.pixbuf = None
        try:
            icon = obj.getIcon()
#            image = gtk.Image()
#            self.pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(\
#                        xdg.IconTheme.getIconPath(icon, 24), 24, 24)
#            image.set_from_pixbuf(self.pixbuf)
            image = panel.get_image_from_file(xdg.IconTheme.getIconPath(icon, panel.iconsize))
            image.show()
            self.pixbuf = panel.get_image_from_file(xdg.IconTheme.getIconPath(icon, 24), 24).get_pixbuf()
        except Exception, e:
            print e
            image = panel.get_stock_image(gtk.STOCK_EXECUTE)
            image.show()
            self.pixbuf = None
        mybox = gtk.HBox(False, 4)
        mybox.show()
        self.add(mybox)
        mybox.pack_start(image, False, True)
        #self.add(image)
        try:
            name = obj.getName()
        except:
            name = ""
        try:
            comment = obj.getComment()
        except:
            comment = ""
        self.menu_name = name.replace('&', '&amp;')
        self.menu_description = comment.replace('&', '&amp;')
        self.items = barpanel.BarpanelMenuWindow(self.menu_name, self.menu_description, self.pixbuf, panel)

        if self.plugin.settings!=None and self.plugin.settings.has_key('labels') and self.plugin.settings['labels'] == 'yes':
            lbl = gtk.Label("")
            panel.update_font(plugin, lbl)
            lbl.set_markup(self.menu_name)
            lbl.set_alignment(0.0, 0.5)
            mybox.pack_start(lbl, True, True)

        if self.plugin.settings.has_key("edit_command"):
            btn = gtk.Button()
            btn.set_name("EdgeButton")
            btn.connect("clicked", self.onEditMenus)
            btn.set_relief(gtk.RELIEF_NONE)
            box = gtk.HBox(False, 2)
            btn.add(box)
            img = panel.get_stock_image(gtk.STOCK_PREFERENCES, 24)
            box.add(img)
            btn.show_all()
            barpanel.tooltips.set_tip(btn, _("Edit Menus"))
            self.items.header.pack_end(btn, False)

        for i in obj.getEntries():
            ib = None
            if isinstance(i, xdg.Menu.Separator):
                ib = separator()
            elif hasattr(i, "DesktopEntry"):
                ib = item(i.DesktopEntry)
                if len(i.DesktopEntry.getOnlyShowIn()) > 0:
                    del(ib)
                    continue
                ib.connect("button-release-event", self.cb_clicked)
            if ib:
                self.items.append(ib)

    def onEditMenus(self, btn):
        barpanel.launch_command(self.plugin.settings['edit_command'])
        self.items.close()
        
    def cb_clicked(self, entry, event):
        exp = re.compile("\%[FUfu]{1}")
        if event.button == 1:
            cmd = exp.sub("", entry.info.command, 1)
        elif event.button == 3:
            dialog = gtk.FileChooserDialog(_("Select file to open with %s") %\
                            entry.info.name, 
                            globals.window, gtk.FILE_CHOOSER_ACTION_OPEN,
                            (gtk.STOCK_CANCEL, gtk.RESPONSE_NONE,
                             gtk.STOCK_OPEN, gtk.RESPONSE_ACCEPT))
            dialog.set_current_folder(os.environ["HOME"])
            r = dialog.run()
            if r == gtk.RESPONSE_NONE:
                dialog.destroy()
                self.close()
                return
            path = dialog.get_filename()
            dialog.destroy()
            cmd = exp.sub("%s", entry.info.command, 1) % path
        else:
            return
        self.items.close()
        if entry.info.terminal:
            try:
                term = os.environ["TERMCMD"]
            except:
                try:
                    term = os.environ["TERM"]
                except:
                    term = "xterm"
            cmd = "%s -e %s" % (term, cmd)
        barpanel.launch_command(cmd)

    def toggle(self, widget):
        self.items.toggle(widget)

class Plugin(barpanel.Plugin):
    def __init__(self, panel, settings):
        self.settings = settings
        menuname = "applications.menu"
        if settings!=None and settings.has_key('name'):
            menuname=settings['name']
        barpanel.Plugin.__init__(self, panel, settings)
        box = gtk.VBox(False, 1)
        box.show()
        self.add(box)
        al1 = gtk.Alignment(0.5, 0.5)
        al1.show()
        if self.panel.paneltype == "bottom" or self.panel.paneltype == "top":
            self.menu = gtk.HBox(False, 0)
        else:
            self.menu = gtk.VBox(False, 0)
        self.menu.show()
        al1.add(self.menu)
        box.pack_start(al1, True, True)
#        box.pack_start(self.menu, True, True)
        settings = self.get_settings()
        theme = settings.get_property("gtk-icon-theme-name")
        if theme:
            xdg.Config.setIconTheme(theme)
        for m in xdg.Menu.parse(menuname).getEntries():
            try:
                if isinstance(m, xdg.Menu.Separator):
                    self.logDEBUG("Separator encountered and ignored")
                    continue # we don't support separators
                self.logDEBUG("Reading menu '%s' .." % (m.getName()))
                obj = menu(m, self.panel, self)
            except Exception, e:
                self.logERROR("While reading '%s': '%s'" % (m, e))
                continue
            obj.connect("toggled", obj.toggle)
            barpanel.tooltips.set_tip(obj, "%s - %s" % (obj.menu_name, obj.menu_description))
            self.menu.pack_start(obj, False, True)
